'use strict';

Object.defineProperties(exports, { __esModule: { value: true }, [Symbol.toStringTag]: { value: 'Module' } });

const qwik = require('@builder.io/qwik');
const jsxRuntime = require('@builder.io/qwik/jsx-runtime');
const build = require('@builder.io/qwik/build');
const react = require('react');
const reactDom = require('react-dom');
const client = require('react-dom/client');

const SlotCtx = react.createContext({
    scopeId: ''
});
function main(slotEl, scopeId, RootCmp, props) {
    const newProps = getReactProps(props);
    return mainExactProps(slotEl, scopeId, RootCmp, newProps);
}
function mainExactProps(slotEl, scopeId, RootCmp, props) {
    return react.createElement(SlotCtx.Provider, {
        value: {
            el: slotEl,
            scopeId,
            attachedEl: undefined
        },
        children: react.createElement(RootCmp, {
            ...props,
            children: react.createElement(SlotElement, null)
        })
    });
}
class SlotElement extends react.Component {
    static contextType = SlotCtx;
    slotC = react.createRef();
    shouldComponentUpdate() {
        return false;
    }
    componentDidMount() {
        const slotC = this.slotC.current;
        if (slotC) {
            const { attachedEl , el  } = this.context;
            if (el) {
                if (!attachedEl) slotC.appendChild(el);
                else if (attachedEl !== slotC) throw new Error('already attached');
            }
        }
    }
    render() {
        return react.createElement('q-slotc', {
            class: this.context.scopeId,
            suppressHydrationWarning: true,
            dangerouslySetInnerHTML: {
                __html: '<!--SLOT-->'
            },
            ref: this.slotC
        });
    }
}
const getReactProps = (props)=>{
    const obj = {};
    Object.keys(props).forEach((key)=>{
        if (!key.startsWith('client:') && !key.startsWith(HOST_PREFIX)) {
            const normalizedKey = key.endsWith('$') ? key.slice(0, -1) : key;
            obj[normalizedKey] = props[key];
        }
    });
    return obj;
};
const getHostProps = (props)=>{
    const obj = {};
    Object.keys(props).forEach((key)=>{
        if (key.startsWith(HOST_PREFIX)) obj[key.slice(HOST_PREFIX.length)] = props[key];
    });
    return obj;
};
const useWakeupSignal = (props, opts = {})=>{
    const signal = qwik.useSignal(false);
    const activate = qwik.inlinedQrl(()=>{
        const [signal] = qwik.useLexicalScope();
        return signal.value = true;
    }, "useWakeupSignal_activate_6LYztwGzxAA", [
        signal
    ]);
    const clientOnly = !!(props['client:only'] || opts?.clientOnly);
    if (build.isServer) {
        if (props['client:visible'] || opts?.eagerness === 'visible') qwik.useOn('qvisible', activate);
        if (props['client:idle'] || opts?.eagerness === 'idle') qwik.useOnDocument('qidle', activate);
        if (props['client:load'] || clientOnly || opts?.eagerness === 'load') qwik.useOnDocument('qinit', activate);
        if (props['client:hover'] || opts?.eagerness === 'hover') qwik.useOn('mouseover', activate);
        if (props['client:event']) qwik.useOn(props['client:event'], activate);
        if (opts?.event) qwik.useOn(opts?.event, activate);
    }
    return [
        signal,
        clientOnly
    ];
};
const HOST_PREFIX = 'host:';

async function renderFromServer(Host, reactCmp$, scopeId, props, ref, slotRef, hydrationProps) {
    const [Cmp, server] = await Promise.all([
        reactCmp$.resolve(),
        Promise.resolve().then(() => require('./server.9d20470e.cjs'))
    ]);
    const render = server.renderToString;
    const newProps = getReactProps(props);
    Object.assign(hydrationProps, newProps);
    const html = render(mainExactProps(undefined, scopeId, Cmp, newProps));
    const index = html.indexOf('<!--SLOT-->');
    if (index > 0) {
        const part1 = html.slice(0, index);
        const part2 = html.slice(index + 11);
        return /*#__PURE__*/ jsxRuntime.jsx(Host, {
            ref: ref,
            ...getHostProps(props),
            children: /*#__PURE__*/ jsxRuntime.jsx(qwik.SSRStream, {
                children: async function*() {
                    yield /*#__PURE__*/ jsxRuntime.jsx(qwik.SSRRaw, {
                        data: part1
                    });
                    yield /*#__PURE__*/ jsxRuntime.jsx("q-slot", {
                        ref: slotRef,
                        children: /*#__PURE__*/ jsxRuntime.jsx(qwik.Slot, {})
                    });
                    yield /*#__PURE__*/ jsxRuntime.jsx(qwik.SSRRaw, {
                        data: part2
                    });
                }
            })
        });
    }
    return /*#__PURE__*/ jsxRuntime.jsxs(jsxRuntime.Fragment, {
        children: [
            /*#__PURE__*/ jsxRuntime.jsx(Host, {
                ref: ref,
                children: /*#__PURE__*/ jsxRuntime.jsx(qwik.SSRRaw, {
                    data: html,
                    [qwik._IMMUTABLE]: {
                        data: true
                    }
                })
            }),
            /*#__PURE__*/ jsxRuntime.jsx("q-slot", {
                ref: slotRef,
                children: /*#__PURE__*/ jsxRuntime.jsx(qwik.Slot, {})
            })
        ]
    });
}

function qwikifyQrl(reactCmp$, opts) {
    return /*#__PURE__*/ qwik.componentQrl(qwik.inlinedQrl((props)=>{
        const [opts, reactCmp$] = qwik.useLexicalScope();
        const { scopeId  } = qwik.useStylesScopedQrl(qwik.inlinedQrl(`q-slot{display:none} q-slotc,q-slotc>q-slot{display:contents}`, "qwikifyQrl_component_useStylesScoped_bHjzplUVBNo"));
        const hostRef = qwik.useSignal();
        const slotRef = qwik.useSignal();
        const internalState = qwik.useSignal();
        const [signal, isClientOnly] = useWakeupSignal(props, opts);
        const hydrationKeys = {};
        const TagName = opts?.tagName ?? 'qwik-react';
        // Watch takes cares of updates and partial hydration
        qwik.useWatchQrl(qwik.inlinedQrl(async ({ track  })=>{
            const [hostRef, hydrationKeys, internalState, props, reactCmp$, scopeId, signal, slotRef] = qwik.useLexicalScope();
            const trackedProps = track(()=>({
                    ...props
                }));
            track(signal);
            if (!build.isBrowser) return;
            // Update
            if (internalState.value) {
                if (internalState.value.root) internalState.value.root.render(main(slotRef.value, scopeId, internalState.value.cmp, trackedProps));
            } else {
                let root = undefined;
                const Cmp = await reactCmp$.resolve();
                const hostElement = hostRef.value;
                if (hostElement) {
                    // hydration
                    root = reactDom.flushSync(()=>{
                        return client.hydrateRoot(hostElement, mainExactProps(slotRef.value, scopeId, Cmp, hydrationKeys));
                    });
                    if (signal.value === false) root.render(main(slotRef.value, scopeId, Cmp, trackedProps));
                }
                internalState.value = qwik.noSerialize({
                    cmp: Cmp,
                    root
                });
            }
        }, "qwikifyQrl_component_useWatch_x04JC5xeP1U", [
            hostRef,
            hydrationKeys,
            internalState,
            props,
            reactCmp$,
            scopeId,
            signal,
            slotRef
        ]));
        if (build.isServer && !isClientOnly) {
            const jsx = renderFromServer(TagName, reactCmp$, scopeId, props, hostRef, slotRef, hydrationKeys);
            return /*#__PURE__*/ jsxRuntime.jsx(qwik.RenderOnce, {
                children: jsx
            });
        }
        return /*#__PURE__*/ jsxRuntime.jsxs(qwik.RenderOnce, {
            children: [
                /*#__PURE__*/ jsxRuntime.jsx(TagName, {
                    ...getHostProps(props),
                    ref: (el)=>{
                        queueMicrotask(()=>{
                            const internalData = internalState.value;
                            if (internalData && !internalData.root) {
                                const root = internalData.root = client.createRoot(el);
                                root.render(main(slotRef.value, scopeId, internalData.cmp, props));
                            }
                        });
                    },
                    children: qwik.SkipRender,
                    [qwik._IMMUTABLE]: {
                        ref: true
                    }
                }),
                /*#__PURE__*/ jsxRuntime.jsx("q-slot", {
                    ref: slotRef,
                    children: /*#__PURE__*/ jsxRuntime.jsx(qwik.Slot, {})
                })
            ]
        });
    }, "qwikifyQrl_component_zH94hIe0Ick", [
        opts,
        reactCmp$
    ]));
}
const qwikify$ = /*#__PURE__*/ qwik.implicit$FirstArg(qwikifyQrl);

function qwikReact() {
    const OPTIMIZE_DEPS = [
        'react',
        'react-dom',
        'react-dom/client',
        'react/jsx-runtime',
        'react/jsx-dev-runtime'
    ];
    const DEDUPE = [
        'react',
        'react-dom'
    ];
    return {
        name: 'vite-plugin-qwik-react',
        config () {
            return {
                resolve: {
                    dedupe: DEDUPE
                },
                optimizeDeps: {
                    include: OPTIMIZE_DEPS
                }
            };
        }
    };
}

exports.qwikReact = qwikReact;
exports.qwikify$ = qwikify$;
exports.qwikifyQrl = qwikifyQrl;
