import { useSignal, inlinedQrl, useLexicalScope, useOn, useOnDocument, SSRStream, SSRRaw, Slot, _IMMUTABLE, componentQrl, useStylesScopedQrl, useWatchQrl, noSerialize, RenderOnce, SkipRender, implicit$FirstArg } from '@builder.io/qwik';
import { jsx, jsxs, Fragment } from '@builder.io/qwik/jsx-runtime';
import { isServer, isBrowser } from '@builder.io/qwik/build';
import { createContext, createElement, Component, createRef } from 'react';
import { flushSync } from 'react-dom';
import { hydrateRoot, createRoot } from 'react-dom/client';

const SlotCtx = createContext({
    scopeId: ''
});
function main(slotEl, scopeId, RootCmp, props) {
    const newProps = getReactProps(props);
    return mainExactProps(slotEl, scopeId, RootCmp, newProps);
}
function mainExactProps(slotEl, scopeId, RootCmp, props) {
    return createElement(SlotCtx.Provider, {
        value: {
            el: slotEl,
            scopeId,
            attachedEl: undefined
        },
        children: createElement(RootCmp, {
            ...props,
            children: createElement(SlotElement, null)
        })
    });
}
class SlotElement extends Component {
    static contextType = SlotCtx;
    slotC = createRef();
    shouldComponentUpdate() {
        return false;
    }
    componentDidMount() {
        const slotC = this.slotC.current;
        if (slotC) {
            const { attachedEl , el  } = this.context;
            if (el) {
                if (!attachedEl) slotC.appendChild(el);
                else if (attachedEl !== slotC) throw new Error('already attached');
            }
        }
    }
    render() {
        return createElement('q-slotc', {
            class: this.context.scopeId,
            suppressHydrationWarning: true,
            dangerouslySetInnerHTML: {
                __html: '<!--SLOT-->'
            },
            ref: this.slotC
        });
    }
}
const getReactProps = (props)=>{
    const obj = {};
    Object.keys(props).forEach((key)=>{
        if (!key.startsWith('client:') && !key.startsWith(HOST_PREFIX)) {
            const normalizedKey = key.endsWith('$') ? key.slice(0, -1) : key;
            obj[normalizedKey] = props[key];
        }
    });
    return obj;
};
const getHostProps = (props)=>{
    const obj = {};
    Object.keys(props).forEach((key)=>{
        if (key.startsWith(HOST_PREFIX)) obj[key.slice(HOST_PREFIX.length)] = props[key];
    });
    return obj;
};
const useWakeupSignal = (props, opts = {})=>{
    const signal = useSignal(false);
    const activate = inlinedQrl(()=>{
        const [signal] = useLexicalScope();
        return signal.value = true;
    }, "useWakeupSignal_activate_6LYztwGzxAA", [
        signal
    ]);
    const clientOnly = !!(props['client:only'] || opts?.clientOnly);
    if (isServer) {
        if (props['client:visible'] || opts?.eagerness === 'visible') useOn('qvisible', activate);
        if (props['client:idle'] || opts?.eagerness === 'idle') useOnDocument('qidle', activate);
        if (props['client:load'] || clientOnly || opts?.eagerness === 'load') useOnDocument('qinit', activate);
        if (props['client:hover'] || opts?.eagerness === 'hover') useOn('mouseover', activate);
        if (props['client:event']) useOn(props['client:event'], activate);
        if (opts?.event) useOn(opts?.event, activate);
    }
    return [
        signal,
        clientOnly
    ];
};
const HOST_PREFIX = 'host:';

async function renderFromServer(Host, reactCmp$, scopeId, props, ref, slotRef, hydrationProps) {
    const [Cmp, server] = await Promise.all([
        reactCmp$.resolve(),
        import('./server.f5cf7d4a.js')
    ]);
    const render = server.renderToString;
    const newProps = getReactProps(props);
    Object.assign(hydrationProps, newProps);
    const html = render(mainExactProps(undefined, scopeId, Cmp, newProps));
    const index = html.indexOf('<!--SLOT-->');
    if (index > 0) {
        const part1 = html.slice(0, index);
        const part2 = html.slice(index + 11);
        return /*#__PURE__*/ jsx(Host, {
            ref: ref,
            ...getHostProps(props),
            children: /*#__PURE__*/ jsx(SSRStream, {
                children: async function*() {
                    yield /*#__PURE__*/ jsx(SSRRaw, {
                        data: part1
                    });
                    yield /*#__PURE__*/ jsx("q-slot", {
                        ref: slotRef,
                        children: /*#__PURE__*/ jsx(Slot, {})
                    });
                    yield /*#__PURE__*/ jsx(SSRRaw, {
                        data: part2
                    });
                }
            })
        });
    }
    return /*#__PURE__*/ jsxs(Fragment, {
        children: [
            /*#__PURE__*/ jsx(Host, {
                ref: ref,
                children: /*#__PURE__*/ jsx(SSRRaw, {
                    data: html,
                    [_IMMUTABLE]: {
                        data: true
                    }
                })
            }),
            /*#__PURE__*/ jsx("q-slot", {
                ref: slotRef,
                children: /*#__PURE__*/ jsx(Slot, {})
            })
        ]
    });
}

function qwikifyQrl(reactCmp$, opts) {
    return /*#__PURE__*/ componentQrl(inlinedQrl((props)=>{
        const [opts, reactCmp$] = useLexicalScope();
        const { scopeId  } = useStylesScopedQrl(inlinedQrl(`q-slot{display:none} q-slotc,q-slotc>q-slot{display:contents}`, "qwikifyQrl_component_useStylesScoped_bHjzplUVBNo"));
        const hostRef = useSignal();
        const slotRef = useSignal();
        const internalState = useSignal();
        const [signal, isClientOnly] = useWakeupSignal(props, opts);
        const hydrationKeys = {};
        const TagName = opts?.tagName ?? 'qwik-react';
        // Watch takes cares of updates and partial hydration
        useWatchQrl(inlinedQrl(async ({ track  })=>{
            const [hostRef, hydrationKeys, internalState, props, reactCmp$, scopeId, signal, slotRef] = useLexicalScope();
            const trackedProps = track(()=>({
                    ...props
                }));
            track(signal);
            if (!isBrowser) return;
            // Update
            if (internalState.value) {
                if (internalState.value.root) internalState.value.root.render(main(slotRef.value, scopeId, internalState.value.cmp, trackedProps));
            } else {
                let root = undefined;
                const Cmp = await reactCmp$.resolve();
                const hostElement = hostRef.value;
                if (hostElement) {
                    // hydration
                    root = flushSync(()=>{
                        return hydrateRoot(hostElement, mainExactProps(slotRef.value, scopeId, Cmp, hydrationKeys));
                    });
                    if (signal.value === false) root.render(main(slotRef.value, scopeId, Cmp, trackedProps));
                }
                internalState.value = noSerialize({
                    cmp: Cmp,
                    root
                });
            }
        }, "qwikifyQrl_component_useWatch_x04JC5xeP1U", [
            hostRef,
            hydrationKeys,
            internalState,
            props,
            reactCmp$,
            scopeId,
            signal,
            slotRef
        ]));
        if (isServer && !isClientOnly) {
            const jsx$1 = renderFromServer(TagName, reactCmp$, scopeId, props, hostRef, slotRef, hydrationKeys);
            return /*#__PURE__*/ jsx(RenderOnce, {
                children: jsx$1
            });
        }
        return /*#__PURE__*/ jsxs(RenderOnce, {
            children: [
                /*#__PURE__*/ jsx(TagName, {
                    ...getHostProps(props),
                    ref: (el)=>{
                        queueMicrotask(()=>{
                            const internalData = internalState.value;
                            if (internalData && !internalData.root) {
                                const root = internalData.root = createRoot(el);
                                root.render(main(slotRef.value, scopeId, internalData.cmp, props));
                            }
                        });
                    },
                    children: SkipRender,
                    [_IMMUTABLE]: {
                        ref: true
                    }
                }),
                /*#__PURE__*/ jsx("q-slot", {
                    ref: slotRef,
                    children: /*#__PURE__*/ jsx(Slot, {})
                })
            ]
        });
    }, "qwikifyQrl_component_zH94hIe0Ick", [
        opts,
        reactCmp$
    ]));
}
const qwikify$ = /*#__PURE__*/ implicit$FirstArg(qwikifyQrl);

function qwikReact() {
    const OPTIMIZE_DEPS = [
        'react',
        'react-dom',
        'react-dom/client',
        'react/jsx-runtime',
        'react/jsx-dev-runtime'
    ];
    const DEDUPE = [
        'react',
        'react-dom'
    ];
    return {
        name: 'vite-plugin-qwik-react',
        config () {
            return {
                resolve: {
                    dedupe: DEDUPE
                },
                optimizeDeps: {
                    include: OPTIMIZE_DEPS
                }
            };
        }
    };
}

export { qwikReact, qwikify$, qwikifyQrl };
